/***************************************
 *                                     *
 *   LCDCLOCK.C                        *
 *                                     *
 *   Version 1.2 C Source Code         *
 *   Copyright (c) 1995 Charles Masar  *
 *                                     *
 ***************************************/


#include <windows.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include "lcdclock.h"

                      
long FAR PASCAL _export ClockWndProc (HWND, UINT, UINT, LONG);
BOOL FAR PASCAL _export OptionsDlgProc (HWND, UINT, UINT, LONG);
BOOL FAR PASCAL _export AboutDlgProc (HWND, UINT, UINT, LONG);
void SetEdgeBools (void);
void UpdateClock (HDC);
long ReadINIFile (void);
void WriteINIFile (void);


HANDLE      hInst;
HWND        hwndClock, hwndTopOrBottom;
HBITMAP     hbmNumbers, hbmColons, hbmClock[2];
short       nXPos, nYPos, nClockColor;
BOOL        bTop, bBottom, bLeft, bRight;
BOOL        bAlwaysOnTop, bShowSeconds, bShowDate, bSaveOnExit;


int PASCAL WinMain (HANDLE hInstance,  HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow)
{
    char        szAppName[] = "LCDClock";
    MSG         msg;
    WNDCLASS    wndclass;

    if (!hPrevInstance)
    {
        wndclass.style         = CS_DBLCLKS;
        wndclass.lpfnWndProc   = ClockWndProc;
        wndclass.cbClsExtra    = 0;
        wndclass.cbWndExtra    = 0;
        wndclass.hInstance     = hInstance;
        wndclass.hIcon         = LoadIcon (hInstance, szAppName);
        wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
        wndclass.hbrBackground = GetStockObject (BLACK_BRUSH);
        wndclass.lpszMenuName  = NULL;
        wndclass.lpszClassName = szAppName;
        RegisterClass (&wndclass);
    }
    hInst = hInstance;

    hwndClock = CreateWindowEx (ReadINIFile(), szAppName, szAppName,
                                WS_POPUP | WS_BORDER, nXPos, nYPos, 
                                bShowSeconds ? CLOCK_LWIDTH : CLOCK_SWIDTH,
                                CLOCK_HEIGHT, NULL, NULL, hInst, NULL);

    while (!SetTimer (hwndClock, IDT_CLOCK, TIMEOUT, NULL))
        if (MessageBox (hwndClock, "Too many clocks or timers!",
                        "LCD Clock", MB_ICONEXCLAMATION | MB_RETRYCANCEL) 
                        == IDCANCEL) return FALSE;
                        
    ShowWindow (hwndClock, nCmdShow);
    UpdateWindow (hwndClock);

    while (GetMessage (&msg, NULL, 0, 0))
    {
        TranslateMessage (&msg);
        DispatchMessage (&msg);
    }
    return msg.wParam;
}


long FAR PASCAL _export ClockWndProc (HWND hwnd,   UINT message,
                                      UINT wParam, LONG lParam)
{
    HDC             hdc;
    PAINTSTRUCT     ps;
    RECT            rect;
    static FARPROC  lpfnOptDlg;

    switch (message)
    {
        case WM_CREATE:
            lpfnOptDlg = MakeProcInstance ((FARPROC) OptionsDlgProc, hInst);
            hbmNumbers = LoadBitmap (hInst, MAKEINTRESOURCE (IDB_NUMBERS)); 
            hbmColons = LoadBitmap (hInst, MAKEINTRESOURCE (IDB_COLONS)); 
            hbmClock[0] = LoadBitmap (hInst, MAKEINTRESOURCE (IDB_SCLOCK));
            hbmClock[1] = LoadBitmap (hInst, MAKEINTRESOURCE (IDB_LCLOCK));
            return 0;

        case WM_PAINT:
            hdc = BeginPaint (hwndClock, &ps);
            UpdateClock (hdc);
            EndPaint (hwndClock, &ps);
            return 0;
        
        case WM_TIMER:
            if (bShowDate) return 0;
            InvalidateRect (hwndClock, NULL, FALSE);
            return 0;

        case WM_LBUTTONDOWN:
            SendMessage (hwndClock, WM_SYSCOMMAND, 0xF012, 0L);
            GetWindowRect (hwndClock, &rect);
            nXPos = rect.left;
            nYPos = rect.top;
            return 0;
        
        case WM_LBUTTONDBLCLK:
            DialogBox (hInst, MAKEINTRESOURCE (IDD_OPTIONS), hwnd, lpfnOptDlg);
            return 0;

        case WM_RBUTTONDOWN:
            bShowDate = !bShowDate;
            InvalidateRect (hwndClock, NULL, FALSE);
            return 0;

        case WM_DESTROY:
            KillTimer (hwndClock, IDT_CLOCK);
            DeleteObject (hbmNumbers); 
            DeleteObject (hbmColons); 
            DeleteObject (hbmClock[0]);
            DeleteObject (hbmClock[1]);
            if (bSaveOnExit) WriteINIFile();
            PostQuitMessage (0);
            return 0;
    }
    return DefWindowProc (hwnd, message, wParam, lParam);
}


BOOL FAR PASCAL _export OptionsDlgProc (HWND hwnd,   UINT message,
                                        UINT wParam, LONG lParam)
{
    static BOOL     bOldAlwaysOnTop, bOldShowSeconds;
    static BOOL     bOldShowDate, bOldSaveOnExit;
    static short    nOldClockColor, nOldXPos, nOldYPos;
    static short    nClockWidth, nOldClockWidth;
    static FARPROC  lpfnAbout;

    switch (message)
    {
    case WM_INITDIALOG:
        lpfnAbout = MakeProcInstance ((FARPROC) AboutDlgProc, hInst);

        nOldClockColor      = nClockColor;
        bOldAlwaysOnTop     = bAlwaysOnTop;
        bOldShowSeconds     = bShowSeconds;
        bOldShowDate        = bShowDate;
        bOldSaveOnExit      = bSaveOnExit;
        nOldXPos            = nXPos;
        nOldYPos            = nYPos;

        SetEdgeBools();
        
        nClockWidth = bShowSeconds ? CLOCK_LWIDTH : CLOCK_SWIDTH;
        nOldClockWidth = nClockWidth;
        
        if (bTop) CheckRadioButton (hwnd, IDC_TOP, IDC_BOTTOM, IDC_TOP);
        if (bBottom) CheckRadioButton (hwnd, IDC_TOP, IDC_BOTTOM, IDC_BOTTOM);
        if (bLeft) CheckRadioButton (hwnd, IDC_LEFT, IDC_RIGHT, IDC_LEFT);
        if (bRight) CheckRadioButton (hwnd, IDC_LEFT, IDC_RIGHT, IDC_RIGHT);
        
        CheckRadioButton (hwnd, IDC_RED, IDC_GRAY,
                          nClockColor + (IDC_RED - RED));
                          
        CheckDlgButton (hwnd, IDC_TOPMOST, bAlwaysOnTop);
        CheckDlgButton (hwnd, IDC_SECONDS, bShowSeconds);
        CheckDlgButton (hwnd, IDC_DATE, bShowDate);
        CheckDlgButton (hwnd, IDC_ONEXIT, bSaveOnExit);
        return TRUE;

    case WM_COMMAND:
        switch (wParam)
        {
        case IDC_TOP:
        case IDC_BOTTOM:
            CheckRadioButton (hwnd, IDC_TOP, IDC_BOTTOM, wParam);
            if (wParam == IDC_TOP) nYPos = 0;
            else nYPos = GetSystemMetrics (SM_CYSCREEN) - CLOCK_HEIGHT;
            SetEdgeBools();        
            MoveWindow (hwndClock, nXPos, nYPos, nClockWidth, 
                        CLOCK_HEIGHT, TRUE);
            UpdateWindow (hwndClock);
            BringWindowToTop (hwnd);
            return TRUE;
        
        case IDC_LEFT:
        case IDC_RIGHT:
            CheckRadioButton (hwnd, IDC_LEFT, IDC_RIGHT, wParam);
            if (wParam == IDC_LEFT) nXPos = 0;
            else nXPos = GetSystemMetrics (SM_CXSCREEN) - nClockWidth;
            SetEdgeBools();
            MoveWindow (hwndClock, nXPos, nYPos, nClockWidth, 
                        CLOCK_HEIGHT, TRUE);
            UpdateWindow (hwndClock);
            BringWindowToTop (hwnd);
            return TRUE;
     
        case IDC_RED:
        case IDC_GREEN:
        case IDC_BLUE:
        case IDC_GRAY:
            CheckRadioButton (hwnd, IDC_RED, IDC_GRAY, wParam);
            nClockColor = wParam - IDC_RED;
            InvalidateRect (hwndClock, NULL, FALSE);
            UpdateWindow (hwndClock);
            BringWindowToTop (hwnd);
            return TRUE;

        case IDC_TOPMOST:
            bAlwaysOnTop = !bAlwaysOnTop;
            if (bAlwaysOnTop == TRUE) hwndTopOrBottom = HWND_TOPMOST;
            else hwndTopOrBottom = HWND_NOTOPMOST;

            CheckDlgButton (hwnd, IDC_TOPMOST, bAlwaysOnTop);
            SetWindowPos (hwndClock, hwndTopOrBottom, 0, 0, 0, 0,
                          SWP_NOMOVE | SWP_NOSIZE);
            BringWindowToTop (hwnd);
            return TRUE;

        case IDC_SECONDS:
            bShowSeconds = !bShowSeconds;
            CheckDlgButton (hwnd, IDC_SECONDS, bShowSeconds);
            nClockWidth = bShowSeconds ? CLOCK_LWIDTH : CLOCK_SWIDTH;
            if (!bShowSeconds && bRight)
                nXPos = GetSystemMetrics (SM_CXSCREEN) - nClockWidth;
            if ((nXPos + nClockWidth) >= GetSystemMetrics (SM_CXSCREEN))
            {
                nXPos = GetSystemMetrics (SM_CXSCREEN) - nClockWidth;
                bRight = TRUE;
                CheckRadioButton (hwnd, IDC_LEFT, IDC_RIGHT, IDC_RIGHT);
            }
            MoveWindow (hwndClock, nXPos, nYPos, nClockWidth, 
                        CLOCK_HEIGHT, TRUE);
            InvalidateRect (hwndClock, NULL, TRUE);            
            UpdateWindow (hwndClock);
            BringWindowToTop (hwnd);
            return TRUE;

        case IDC_DATE:
            bShowDate = !bShowDate;
            CheckDlgButton (hwnd, IDC_DATE, bShowDate);
            InvalidateRect (hwndClock, NULL, FALSE);
            UpdateWindow (hwndClock);
            BringWindowToTop (hwnd);
            return TRUE;

        case IDC_ONEXIT:
            bSaveOnExit = !bSaveOnExit;
            CheckDlgButton (hwnd, IDC_ONEXIT, bSaveOnExit);
            return TRUE;
        
        case IDC_SAVE:
            WriteINIFile();
            return TRUE;

        case IDC_CLOSE:
            EndDialog (hwnd, TRUE);
            WinHelp (hwnd, "lcdclock.hlp", HELP_QUIT, 0L);
            SendMessage (hwndClock, WM_CLOSE, 0, 0L);
            return TRUE;

        case IDC_ABOUT:
            DialogBox (hInst, MAKEINTRESOURCE (IDD_ABOUT), hwnd, lpfnAbout);
            return TRUE;

        case IDC_OK:
            WinHelp (hwnd, "lcdclock.hlp", HELP_QUIT, 0L);
            EndDialog (hwnd, TRUE);
            return TRUE;

        case IDC_HELP:
            WinHelp (hwnd, "lcdclock.hlp", HELP_CONTENTS, 0L);
            return TRUE;
        
        case IDC_CANCEL:
            nClockColor     = nOldClockColor;
            bAlwaysOnTop    = bOldAlwaysOnTop;
            bShowSeconds    = bOldShowSeconds;
            bShowDate       = bOldShowDate;
            bSaveOnExit     = bOldSaveOnExit;
            nXPos           = nOldXPos;
            nYPos           = nOldYPos;
            nClockWidth     = nOldClockWidth; 
            
            if (bAlwaysOnTop == TRUE) hwndTopOrBottom = HWND_TOPMOST;
            else hwndTopOrBottom = HWND_NOTOPMOST;
            SetWindowPos (hwndClock, hwndTopOrBottom, nOldXPos, nOldYPos, 
                          nClockWidth, CLOCK_HEIGHT, NULL);
            InvalidateRect (hwndClock, NULL, FALSE);
            WinHelp (hwnd, "lcdclock.hlp", HELP_QUIT, 0L);
            EndDialog (hwnd, TRUE);
            return TRUE;
        }
        break;

    case WM_CLOSE:
        EndDialog (hwnd, 0);
        WinHelp (hwnd, "lcdclock.hlp", HELP_QUIT, 0L);
        return TRUE;
    }
    return FALSE;
}


BOOL FAR PASCAL _export AboutDlgProc (HWND hwnd,   UINT message,
                                      UINT wParam, LONG lParam)
{
    switch (message)
    {
    case WM_COMMAND:
        if (wParam == IDC_OK)
        {
            EndDialog (hwnd, TRUE);
            return TRUE;
        }
        break;

    case WM_CLOSE:
        EndDialog (hwnd, TRUE);
        return TRUE;
    }
    return FALSE;
}


void SetEdgeBools (void)
{   
    int     nClockWidth;

    bTop    = FALSE;
    bLeft   = FALSE;
    bBottom = FALSE;
    bRight  = FALSE;
    
    nClockWidth = bShowSeconds ? CLOCK_LWIDTH : CLOCK_SWIDTH;
    
    if (nYPos == 0) bTop  = TRUE;
    if (nXPos == 0) bLeft = TRUE;
    if (nYPos == GetSystemMetrics (SM_CYSCREEN) - CLOCK_HEIGHT) bBottom = TRUE;
    if (nXPos == GetSystemMetrics (SM_CXSCREEN) - nClockWidth) bRight = TRUE;
}


void UpdateClock (HDC hdc)
{   
    HDC             hdcClock, hdcNumbers, hdcColons;
    static short    num1, num2, num3, num4, num5, num6;
    static int      colon = 1;
    struct tm       *tmTime;
    time_t          lTime;

    time (&lTime);
    tmTime = localtime (&lTime);

    if (bShowDate)
    {
        colon = 1;
        tmTime->tm_mon += 1;
        num1 = (tmTime->tm_mon)  / 10;
        num2 = (tmTime->tm_mon)  - (num1 * 10);
        num3 = (tmTime->tm_mday) / 10;
        num4 = (tmTime->tm_mday) - (num3 * 10);
        num5 = (tmTime->tm_year) / 10;
        num6 = (tmTime->tm_year) - (num5 * 10);
        if (num1 == 0)  num1 = 10;
    }
    else
    {   
        colon = !colon;
        if ((tmTime->tm_hour) > 12) (tmTime->tm_hour) -= 12;
        if ((tmTime->tm_hour) == 0) (tmTime->tm_hour) = 12;
        num1 = (tmTime->tm_hour) / 10;
        num2 = (tmTime->tm_hour) - (num1 * 10);
        num3 = (tmTime->tm_min)  / 10;
        num4 = (tmTime->tm_min)  - (num3 * 10);
        num5 = (tmTime->tm_sec)  / 10;
        num6 = (tmTime->tm_sec)  - (num5 * 10);
        if (num1 == 0)  num1 = 10;
    } 
    
    hdcClock = CreateCompatibleDC (hdc);
    hdcNumbers = CreateCompatibleDC (hdc);
    hdcColons = CreateCompatibleDC (hdc);
    
    SelectObject (hdcClock, hbmClock[bShowSeconds]);
    SelectObject (hdcNumbers, hbmNumbers);
    SelectObject (hdcColons, hbmColons);
    
    /* Draw the hour */
    BitBlt (hdcClock, HOUR1_XSTART, NUMBER_YSTART, 
            NUMBER_WIDTH, NUMBER_HEIGHT, hdcNumbers, 
            num1 * NUMBER_WIDTH, nClockColor * NUMBER_HEIGHT, SRCCOPY);
    BitBlt (hdcClock, HOUR2_XSTART, NUMBER_YSTART, 
            NUMBER_WIDTH, NUMBER_HEIGHT, hdcNumbers, 
            num2 * NUMBER_WIDTH, nClockColor * NUMBER_HEIGHT, SRCCOPY);
            
    /* Draw the first colon */
    BitBlt (hdcClock, COLON1_XSTART, NUMBER_YSTART, 
            COLON_WIDTH, COLON_HEIGHT, hdcColons, 
            colon * COLON_WIDTH, nClockColor * COLON_HEIGHT, SRCCOPY);
            
    /* Draw the minutes */        
    BitBlt (hdcClock, MIN1_XSTART, NUMBER_YSTART, 
            NUMBER_WIDTH, NUMBER_HEIGHT, hdcNumbers, 
            num3 * NUMBER_WIDTH, nClockColor * NUMBER_HEIGHT, SRCCOPY);
    BitBlt (hdcClock, MIN2_XSTART, NUMBER_YSTART, 
            NUMBER_WIDTH, NUMBER_HEIGHT, hdcNumbers, 
            num4 * NUMBER_WIDTH, nClockColor * NUMBER_HEIGHT, SRCCOPY);
    
    if (bShowSeconds)
    { 
        /* Draw the second colon */
        BitBlt (hdcClock, COLON2_XSTART, NUMBER_YSTART, 
                COLON_WIDTH, COLON_HEIGHT, hdcColons, 
                colon * COLON_WIDTH, nClockColor * COLON_HEIGHT, SRCCOPY);
                
        /* Draw the seconds */
        BitBlt (hdcClock, SEC1_XSTART, NUMBER_YSTART, 
                NUMBER_WIDTH, NUMBER_HEIGHT, hdcNumbers, 
                num5 * NUMBER_WIDTH, nClockColor * NUMBER_HEIGHT, SRCCOPY);
        BitBlt (hdcClock, SEC2_XSTART, NUMBER_YSTART, 
                NUMBER_WIDTH, NUMBER_HEIGHT, hdcNumbers, 
                num6 * NUMBER_WIDTH, nClockColor * NUMBER_HEIGHT, SRCCOPY);
    }
    
    /* Draw the clock */
    BitBlt (hdc, -1, -1, bShowSeconds ? CLOCK_LWIDTH : CLOCK_SWIDTH, 
            CLOCK_HEIGHT, hdcClock, 0, 0, SRCCOPY); 
            
    DeleteDC (hdcClock);
    DeleteDC (hdcNumbers);
    DeleteDC (hdcColons);
}


long ReadINIFile (void)
{
    char    szBuf[6];
    char    szSect[] = "LCDClock";
    char    szFile[] = "lcdclock.ini";
    long    dwExStyle;
    int     nXStart, nYStart;

    nClockColor     = RED;
    hwndTopOrBottom = HWND_TOPMOST;
    bAlwaysOnTop    = TRUE;
    dwExStyle       = WS_EX_TOPMOST;
    bShowSeconds    = FALSE;
    bShowDate       = FALSE;
    bSaveOnExit     = FALSE;

    GetPrivateProfileString (szSect, "ClockColor", "blue", 
                             szBuf, sizeof(szBuf), szFile);
    if (!_stricmp (szBuf, "red"))   nClockColor = RED;
    if (!_stricmp (szBuf, "green")) nClockColor = GREEN;
    if (!_stricmp (szBuf, "gray"))  nClockColor = GRAY;

    GetPrivateProfileString (szSect, "AlwaysOnTop", "yes", 
                             szBuf, sizeof(szBuf), szFile);
    if (!_stricmp (szBuf, "no"))
    {
        hwndTopOrBottom = HWND_NOTOPMOST;
        bAlwaysOnTop = FALSE;
        dwExStyle = 0x00000000L;
    }

    GetPrivateProfileString (szSect, "ShowSeconds", "no", 
                             szBuf, sizeof(szBuf), szFile);
    if (!_stricmp (szBuf, "yes")) bShowSeconds = TRUE;

    GetPrivateProfileString (szSect, "ShowDate", "no", 
                             szBuf, sizeof(szBuf), szFile);
    if (!_stricmp (szBuf, "yes")) bShowDate = TRUE;

    GetPrivateProfileString (szSect, "SaveOnExit", "no", 
                             szBuf, sizeof(szBuf), szFile);
    if (!_stricmp (szBuf, "yes")) bSaveOnExit = TRUE;
    
    nXStart = GetSystemMetrics (SM_CXSCREEN) 
            - (bShowSeconds ? CLOCK_LWIDTH : CLOCK_SWIDTH);
    nYStart = GetSystemMetrics (SM_CYSCREEN) - CLOCK_HEIGHT;
    nXPos = GetPrivateProfileInt (szSect, "XPos", nXStart, szFile);
    nYPos = GetPrivateProfileInt (szSect, "YPos", nYStart, szFile);

    return (dwExStyle);
}


void WriteINIFile (void)
{
    char    szBuf[7];
    char    szSect[] = "LCDClock";
    char    szFile[] = "lcdclock.ini";

    sprintf (szBuf, "%hi", nXPos);
    WritePrivateProfileString (szSect, "XPos", szBuf, szFile);
    sprintf (szBuf, "%hi", nYPos);
    WritePrivateProfileString (szSect, "YPos", szBuf, szFile);

    if (nClockColor == RED) strcpy (szBuf, "red");
    else if (nClockColor == GREEN) strcpy (szBuf, "green");
    else if (nClockColor == BLUE) strcpy (szBuf, "blue");
    else strcpy (szBuf, "gray");
    WritePrivateProfileString (szSect, "ClockColor", szBuf, szFile);

    if (bAlwaysOnTop == FALSE) strcpy (szBuf, "no");
    else strcpy (szBuf, "yes");
    WritePrivateProfileString (szSect, "AlwaysOnTop", szBuf, szFile);

    if (bShowSeconds == TRUE) strcpy (szBuf, "yes");
    else strcpy (szBuf, "no");
    WritePrivateProfileString (szSect, "ShowSeconds", szBuf, szFile);

    if (bShowDate == TRUE) strcpy (szBuf, "yes");
    else strcpy (szBuf, "no");
    WritePrivateProfileString (szSect, "ShowDate", szBuf, szFile);

    if (bSaveOnExit == TRUE) strcpy (szBuf, "yes");
    else strcpy (szBuf, "no");
    WritePrivateProfileString (szSect, "SaveOnExit", szBuf, szFile);
}
